#!/bin/sh

################################################################################
#    backupCD 1.1.                                                             #
#    Copyright (C) 2003 Luca Bertoncello                                       #
#    Waltherstrae, 11D - 01067 Dresden Deutschland                            #
#    E-Mail: lucabert@lucabert.com                                             #
#                                                                              #
#    This program is free software; you can redistribute it and/or modify      #
#    it under the terms of the GNU General Public License as published by      #
#    the Free Software Foundation; version 2 of the License.                   #
#                                                                              #
#    This program is distributed in the hope that it will be useful,           #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of            #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
#    GNU General Public License for more details.                              #
#                                                                              #
#    You should have received a copy of the GNU General Public License         #
#    along with this program; if not, write to the Free Software               #
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA #
################################################################################

# Default values (you can change them by parameter):
# Device to write to
ID_CD_WRITER=0,6,0
SPEED=4
# Set AUTOEJECT to 1 in order to eject tape at the end of backup process
# Set AUTOEJECT to 0 in order to rewind tape at the end of backup process, but not eject it
AUTOEJECT=1

# Configurazion strings. You don't need to modify these
MAIN_DIR=tmpBackup
BACKUP_FILE_PREFIX=backup
BACKUP_FILE_SUFFIX=.tgz
FILE_BACKUP_DIR=

# Path of the commands. You can change them, with the path in your system.
CMDAWK=/bin/awk
CMDCAT=/bin/cat
CMDCDRECORD=/opt/schily/bin/stable/cdrecord
CMDCP=/bin/cp
CMDDATE=/bin/date
CMDECHO=/bin/echo
CMDEXPR=/usr/bin/expr
CMDMKDIR=/bin/mkdir
CMDMKISOFS=/opt/schily/bin/stable/mkisofs
CMDPERL=/usr/bin/perl
CMDRM=/bin/rm
CMDTAR=/bin/tar

# Please DO NOT modify under this line!!
filesList="$CMDAWK $CMDCAT $CMDCDRECORD $CMDCP $CMDDATE $CMDECHO $CMDEXPR $CMDMKDIR $CMDMKISOFS $CMDPERL $CMDRM $CMDTAR"

# Check all files
function checkFiles
{
  FILESOK=0
  for fName in $filesList; do
    if test ! -e $fName; then
      $CMDECHO "'$fName': file not exist!"
      FILESOK=1
    fi
  done

  return $FILESOK
}

# Searches for temp directory
function searchTmpDir
{
  tmpDirList="/var/tmp /var/temp /usr/tmp /usr/temp /tmp"

  for dir in $tmpDirList; do
    if test -d $dir; then
      FILE_BACKUP_DIR=$dir
    fi
  done
}

# Checks directories to be backupped
function checkDirsFile
{
  FILEINVALID=0
  if test "$*"; then
    for dir in $*; do
      if test ! -d $dir; then
        FILEINVALID=1
        $CMDECHO "     '$dir': invalid directory!!"
      fi
    done
  else
    FILEINVALID=1
  fi

  return $FILEINVALID
}

function backup
{
  if test ${1} && test -f ${1}; then
    DIR_BACKUPPED=`$CMDCAT ${1} | $CMDAWK '{ if(($0) && ($0 !~ /#/)) print $0; }'`
  else
    if test ${1}; then
      $CMDECHO "Invalid backup file!"
    fi
    FILEOK=0
    while test $FILEOK -eq 0
    do
      $CMDECHO -n "Enter file with list of directories to backup: "
      read CONFILE
      if test ! $CONFILE || test ! -f $CONFILE; then
        $CMDECHO "Invalid backup file!"
      else
        FILEOK=1
      fi
    done
    DIR_BACKUPPED=`$CMDCAT $CONFILE | $CMDAWK '{ if(($0) && ($0 !~ /#/)) print $0; }'`
  fi
  $CMDECHO "==> Parsing config file..."
  checkDirsFile $DIR_BACKUPPED
  if test $? -ne 0; then
    $CMDECHO "In order to start backup procedure, you MUST";
    $CMDECHO "create a file with a list of directories, one per line,";
    $CMDECHO "WITHOUT / at the end.";
    $CMDECHO "Program stopped.";
    exit
  fi

  CURDATE=`$CMDDATE +%d_%m_%Y`;
  DIR_BACKUP=$FILE_BACKUP_DIR/$MAIN_DIR/backup$CURDATE

  PAR_STD_MKISOFS="-f -R -r -l -J -VBackup -A$CURDATE -PBackupCD"

  # Calculate session (first or next)
  $CMDECHO "==> Calculating session..."
  tracks=`$CMDCDRECORD -msinfo dev=$ID_CD_WRITER`
  COMP=`$CMDPERL -e "if ( $tracks =~ m/^\d+\,?\d+$|^\,\d+$|^\d+$/ ) { print 'NEXT'; } else { print 'FIRST'; }"`
  if test "$COMP" = 'NEXT'; then
    parMkisofs="-C$tracks -M$ID_CD_WRITER"
    $CMDECHO "   There are session on CD."
  else
    $CMDECHO "   First track on CD."
    parMkisofs=''
  fi

  # Calculate length of directories
  LEN=21
  for DIR in $DIR_BACKUPPED; do
    LEN=`$CMDPERL -e '$dir = $ARGV[0]; $len = $ARGV[1]; $prefix = $ARGV[2]; if((length($dir) + length($prefix)) > $len) { $len = length($dir) + length($prefix); } print $len;' $DIR $LEN $BACKUP_FILE_PREFIX`;
  done

  # Creating backup archive files
  $CMDECHO "==> Creating Backup..."
  $CMDMKDIR $FILE_BACKUP_DIR/$MAIN_DIR
  $CMDMKDIR $DIR_BACKUP

  ERROR=0
  $CMDCP $0 $DIR_BACKUP/$backupFileName
  for file in $DIR_BACKUPPED; do
    nfile=`$CMDECHO $file | sed -e "s/\//\_/g"`
    file=$file/
    backupFileName=$BACKUP_FILE_PREFIX$nfile$BACKUP_FILE_SUFFIX
    pathFileBackup=$DIR_BACKUP/$backupFileName
    $CMDPERL -e '$file = $ARGV[0]; $len = $ARGV[1]; $len += 25; $format = "%-$len"."s"; printf($format, "     Compressing '$file'...");' $file $LEN
    $CMDTAR zcf $pathFileBackup $file 2> /dev/null
    if test $? -eq 0; then
      $CMDECHO "OK"
    else
      $CMDECHO "FAILED!"
      ERROR=1
    fi
  done
  if test $ERROR -ne 0; then
    $CMDECHO "ERROR in compressing files!";
    $CMDECHO "Program stopped.";
    exit
  fi

  $CMDECHO "==> Creating Filesystem..."
  $CMDMKISOFS -o$FILE_BACKUP_DIR/isoImage.raw $PAR_STD_MKISOFS $parMkisofs $FILE_BACKUP_DIR/$MAIN_DIR

  if test $AUTOEJECT -eq 1; then
    EJECTSTR='-eject'
  else
    EJECTSTR=''
  fi
  $CMDECHO "==> Writing on CD..."
  $CMDCDRECORD -v speed=$SPEED dev=$ID_CD_WRITER $EJECTSTR -multi $FILE_BACKUP_DIR/isoImage.raw
  if test $? -eq 0; then
    BACKUPOK=1
  else
    BACKUPOK=0
  fi

  $CMDECHO "==> Removing temp files..."
  $CMDRM -Rf $FILE_BACKUP_DIR/$MAIN_DIR
  $CMDRM -f $FILE_BACKUP_DIR/isoImage.raw

  if test $BACKUPOK -eq 0; then
    $CMDECHO "==> Backup completed successfully!"
  else
    $CMDECHO "==> ERRORS IN BACKUP!!"
  fi
}

# Search temporary directory
searchTmpDir

function help
{
  if test "$AUTOEJECT" = "1"; then
    AUTOEJECTSTR="true";
  else
    AUTOEJECTSTR="false";
  fi
  $CMDECHO "backupCD 1.1."
  $CMDECHO "Script for backup on CD."
  $CMDECHO "By Luca Bertoncello 2003. GPL v.2.0 Licence"
  $CMDECHO "For information about the program and the author:"
  $CMDECHO "               http://www.lucabert.de/"
  $CMDECHO ""
  $CMDECHO "Parameters:"
  $CMDECHO "  help                  Display this help"
  $CMDECHO "  blank                 Format the CD before backing up"
  $CMDECHO "  bklist=<file>         Use <file> for backup list."
  $CMDECHO "                        If not specified, it will be asked."
  $CMDECHO "  dev=<dev>             Use <dev> as device to write to"
  $CMDECHO "                        (default: $ID_CD_WRITER)."
  $CMDECHO "  speed=<speed>         Use speed <speed> to write on CD"
  $CMDECHO "                        (default: $SPEED)."
  $CMDECHO "  eject=<true|false>    Eject tape at the end of backup/restore"
  $CMDECHO "                        process (default: $AUTOEJECTSTR)."
  $CMDECHO "  tmpdir=<dir>          Use <dir> as temporary directory."
  $CMDECHO "                        (Default: $FILE_BACKUP_DIR)."
}

# Check if all files are installed
checkFiles
if test $? -ne 0; then
  $CMDECHO "Program stopped.";
  exit
fi

BLANK=0
for ARG in $*; do
  RECOGNIZED=0
  if test "$ARG" = "help"; then
    help;
    RECOGNIZED=1;
    exit
  fi
  if test "$ARG" = "blank"; then
    BLANK=1
    RECOGNIZED=1;
  fi
  if test "`$CMDECHO $ARG | $CMDAWK '{print substr($0, 1, 7)}'`" = "tmpdir="; then
    FILE_BACKUP_DIR=`$CMDECHO $ARG | $CMDAWK '{print substr($0, 8)}'`;
    RECOGNIZED=1
  fi
  if test "`$CMDECHO $ARG | $CMDAWK '{print substr($0, 1, 7)}'`" = "bklist="; then
    BKLIST=`$CMDECHO $ARG | $CMDAWK '{print substr($0, 8)}'`;
    RECOGNIZED=1
  fi
  if test "`$CMDECHO $ARG | $CMDAWK '{print substr($0, 1, 4)}'`" = "dev="; then
    ID_CD_WRITER=`$CMDECHO $ARG | $CMDAWK '{print substr($0, 5)}'`;
    RECOGNIZED=1
  fi
  if test "`$CMDECHO $ARG | $CMDAWK '{print substr($0, 1, 6)}'`" = "speed="; then
    SPEED=`$CMDECHO $ARG | $CMDAWK '{print substr($0, 7)}'`;
    RECOGNIZED=1
  fi
  if test "`$CMDECHO $ARG | $CMDAWK '{print substr($0, 1, 6)}'`" = "eject="; then
    STRAUTOEJECT=`$CMDECHO $ARG | $CMDAWK '{print substr($0, 7)}'`;
    RECOGNIZED=1;
    if test "$STRAUTOEJECT" = "true" || test "$STRAUTOEJECT" = "1"; then
      AUTOEJECT=1
    elif test "$STRAUTOEJECT" = "false" || test "$STRAUTOEJECT" = "0"; then
      AUTOEJECT=0
    else
      $CMDECHO "Unknown option '$STRAUTOEJECT'. Using default!";
    fi
  fi

  if test $RECOGNIZED -eq 0; then
    $CMDECHO "Unknown command '$ARG'";
    $CMDECHO "Try backupCD.sh help for information.";
    exit
  fi
done

if test ! -d $FILE_BACKUP_DIR; then
  $CMDECHO "ERROR: '$FILE_BACKUP_DIR' - invalid directory!!";
  $CMDECHO "Program stopped.";
  exit
fi

# When asked, formats CD
if test $BLANK = 1; then
  $CMDECHO "==> Formatting CD..."
  $CMDCDRECORD -v speed=$SPEED dev=$ID_CD_WRITER blank=all
fi

# Starts backup
backup $BKLIST
